@@ -1,6 +1,31 @@
 tssem1FEM <- function(Cov, n, cor.analysis=TRUE, model.name=NULL,
                       cluster=NULL, suppressWarnings=TRUE, silent=TRUE,
                       run=TRUE, ...) {
+    
+  ## Function to mark diagonals as NA when all correlation coefficients of the variable are NA
+  ## assuming matrices are symmetric
+  addDiagNA <- function(x) {
+      add_NA <- function(y) {
+          for(i in 1:nrow(y)) {
+              ## All elements in the row except the diagonal are NA
+              if (sum(is.na(y[, i]))==(nrow(y)-1)) y[i,i] <- NA
+          }
+          y
+      }
+      if (is.list(x)) lapply(x, add_NA) else add_NA(x)
+  }  
+  ## Mark the diagonals as NA when all correlation coefficients are NA
+  Cov <- addDiagNA(Cov)
+
+  ## Smooth non-symmetric matrices probably due to rounding
+  ## assuming no input error in the matrices
+  if (is.list(Cov)) {
+      Cov <- lapply(Cov, function(x) {x <- (x+t(x))/2; x})
+  } else {
+      Cov <- (Cov+t(Cov))/2
+  }
+        
+  ## Split the data by cluster  
   if (!is.null(cluster)) {
     data.cluster <- tapply(Cov, cluster, function(x) {x})
     n.cluster <- tapply(n, cluster, function(x) {x})
@@ -56,7 +81,7 @@ tssem1FEM <- function(Cov, n, cor.analysis=TRUE, model.name=NULL,
       S <- mxMatrix(type="Symm", nrow=no.var, ncol=no.var, free=TRUE, values=vech(sv),
                     labels=vech(ps.labels), lbound=lbound, name="S")
     }
-
+      
     ## Index for missing variables: only check the diagonals only!!!
     miss.index <- lapply(Cov, function(x) { is.na(Diag(x)) })
 
@@ -93,10 +118,6 @@ tssem1FEM <- function(Cov, n, cor.analysis=TRUE, model.name=NULL,
         # Expected covariance matrix
         expC <- mxAlgebra(SD %&% S, name="expC", dimnames=list(var.names[!miss.index.i],
                           var.names[!miss.index.i]))
-        # Create mxModel
-        ## model <- mxModel(paste("group",i,sep=""), S, Dmatrix, expC,
-        ##                      mxData(observed=Cov.i, type="cov", numObs=n[i]),
-        ##                      mxMLObjective(expC, dimnames=var.names[!miss.index[[i]]]))
 
         fitFunction <- mxFitFunctionML()
 
@@ -106,26 +127,6 @@ tssem1FEM <- function(Cov, n, cor.analysis=TRUE, model.name=NULL,
         eval(parse(text = g.model))
     }
 
-    ## if (cor.analysis==TRUE) {
-    ##     tssem1.model <- paste("tssem1 <- mxModel(\"", model.name, "\", ", paste("S",
-    ##         1:no.groups, sep = "", collapse = ","), ", ", paste("D", 1:no.groups,
-    ##         sep = "", collapse = ","), ", ", paste("expC", 1:no.groups, sep = "",
-    ##         collapse = ","), ", ", paste("g", 1:no.groups, sep = "", collapse = ","),
-    ##         ", mxAlgebra(", paste("g", 1:no.groups, ".objective", sep = "", collapse = "+"),
-    ##         ", name=\"obj\"), mxAlgebraObjective(\"obj\"))", sep = "")
-    ## } else {
-    ##     tssem1.modelb <- paste("tssem1 <- mxModel(\"", model.name, "\", ", paste("S",
-    ##         1:no.groups, sep = "", collapse = ","), ", ", paste("g", 1:no.groups,
-    ##         sep = "", collapse = ","), ", mxAlgebra(", paste("g", 1:no.groups, ".objective",
-    ##         sep = "", collapse = "+"), ", name=\"obj\"), mxAlgebraObjective(\"obj\"))",
-    ##         sep = "")
-    ## }
-      
-    ## tssem1.model <- paste("tssem1 <- mxModel(\"", model.name, "\", S, ",
-    ##                       paste("g", 1:no.groups, sep = "", collapse = ","),
-    ##                       ", mxAlgebra(", paste("g", 1:no.groups, ".objective", sep = "", collapse = "+"),
-    ##                       ", name=\"obj\"), mxFitFunctionAlgebra(algebra =\"obj\"))", sep = "")
-
     ## Replaced mxFitFunctionAlgebra() with mxFitFunctionMultigroup()  
     tssem1.model <- paste0("tssem1 <- mxModel(\"", model.name, "\", S, ",
                           paste0("g", 1:no.groups, collapse = ","),
@@ -256,36 +257,55 @@ tssem1 <- function(Cov, n, method=c("REM", "FEM"), cor.analysis=TRUE, cluster=NU
 }
 
 ## Known bug: wls() will fall into loop when the Amatrix is zero
-wls <- function(Cov, aCov, n, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, 
+wls <- function(Cov, aCov, n, RAM=NULL, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, 
                 diag.constraints=FALSE, cor.analysis=TRUE, intervals.type=c("z", "LB"), 
                 mx.algebras=NULL, model.name=NULL, suppressWarnings=TRUE, 
                 silent=TRUE, run=TRUE, ...) {
-  if (is.null(Smatrix)) {
-    stop("\"Smatrix\" matrix is not specified.\n")
-  } else {
-    if (is.matrix(Smatrix)) Smatrix <- as.mxMatrix(Smatrix)
-    ## No. of observed and latent variables
-    p <- nrow(Smatrix@values)
-    Smatrix@name <- "Smatrix"
-  }
 
-  if (is.null(Amatrix)) {
-    Amatrix <- as.mxMatrix(matrix(0, nrow=p, ncol=p), name="Amatrix")
-  } else {
-    if (is.matrix(Amatrix)) Amatrix <- as.mxMatrix(Amatrix)
-    Amatrix@name <- "Amatrix"
-  }
+    ## Read RAM first. If it is not specified, read individual matrices
+    if (!is.null(RAM)) {
+        Amatrix <- as.mxMatrix(RAM$A, name="Amatrix")
+        Smatrix <- as.mxMatrix(RAM$S, name="Smatrix")
+        Fmatrix <- as.mxMatrix(RAM$F, name="Fmatrix")
+    } else {
+        if (is.null(Smatrix)) {
+            stop("\"Smatrix\" matrix is not specified.\n")
+        } else if (is.matrix(Smatrix)) {
+            Smatrix <- as.mxMatrix(Smatrix, name="Smatrix")
+        } else {
+            ## Change the name of the input mxMatrix
+            Smatrix@name <- "Smatrix"
+        }
+        
+        ## No. of observed and latent variables
+        p <- nrow(Smatrix@values)  
 
+        if (is.null(Amatrix)) {
+            ## If Amatrix is not specified, use a zero matrix.
+            Amatrix <- as.mxMatrix(matrix(0, nrow=p, ncol=p), name="Amatrix")
+        } else if (is.matrix(Amatrix)) {
+            Amatrix <- as.mxMatrix(Amatrix, name="Amatrix")
+        } else {
+            ## Change the name of the input mxMatrix
+            Amatrix@name <- "Amatrix"
+        }
+
+        if (is.null(Fmatrix)) {
+            ## If Fmatrix is not specified, use an identity matrix.
+            Fmatrix <- as.mxMatrix(Diag(rep(p,1)), name="Fmatrix")
+        } else if (is.matrix(Fmatrix)) {
+            Fmatrix <- as.mxMatrix(Fmatrix, name="Fmatrix")
+        } else {
+            ## Change the name of the input mxMatrix
+            Fmatrix@name <- "Fmatrix"
+        }    
+    }      
+    
   ## CheckRAM
   checkRAM(Amatrix=Amatrix, Smatrix=Smatrix, cor.analysis=cor.analysis)
-  
-  if (is.null(Fmatrix)) {
-    Fmatrix <- as.mxMatrix(Diag(rep(p,1)), name="Fmatrix")
-  } else {
-    if (is.matrix(Fmatrix)) Fmatrix <- as.mxMatrix(Fmatrix)
-    Fmatrix@name <- "Fmatrix"
-  }
 
+  ## No. of observed and latent variables
+  p <- nrow(Smatrix@values)      
   ## A pxp identity matrix
   Id <- as.mxMatrix(Diag(rep(p, 1)), name="Id")
 
@@ -496,17 +516,18 @@ wls <- function(Cov, aCov, n, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL,
 }
 
 
-tssem2 <- function(tssem1.obj, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, diag.constraints=FALSE,
+tssem2 <- function(tssem1.obj, RAM=NULL, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, diag.constraints=FALSE,
                    intervals.type = c("z", "LB"), mx.algebras=NULL, model.name=NULL, suppressWarnings=TRUE,
                    silent=TRUE, run=TRUE, ...) {
   if ( !is.element( class(tssem1.obj)[1], c("tssem1FEM.cluster", "tssem1FEM", "tssem1REM")) )
       stop("\"tssem1.obj\" must be of neither class \"tssem1FEM.cluster\", class \"tssem1FEM\" or \"tssem1REM\".")
 
   switch(class(tssem1.obj)[1],
-         tssem1FEM.cluster = { out <- lapply(tssem1.obj, tssem2, Amatrix=Amatrix, Smatrix=Smatrix, Fmatrix=Fmatrix,
+         tssem1FEM.cluster = { out <- lapply(tssem1.obj, tssem2, RAM=RAM, Amatrix=Amatrix, Smatrix=Smatrix, Fmatrix=Fmatrix,
                                              diag.constraints=diag.constraints, intervals.type=intervals.type,
                                              mx.algebras=mx.algebras,
-                                             model.name=model.name, suppressWarnings=suppressWarnings, silent=silent, run=run, ...)
+                                             model.name=model.name, suppressWarnings=suppressWarnings, silent=silent,
+                                             run=run, ...)
                               class(out) <- "wls.cluster" },
          tssem1FEM = { if (tssem1.obj$cor.analysis==TRUE) {
                         if (is.null(model.name)) model.name <- "TSSEM2 Correlation"
@@ -519,7 +540,7 @@ tssem2 <- function(tssem1.obj, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, diag.co
                       pooledS <- coef.tssem1FEM(tssem1.obj)                      
                       ## dimnames(pooledS) <- list(tssem1.obj$original.names, tssem1.obj$original.names)
                       out <- wls(Cov=coef.tssem1FEM(tssem1.obj), aCov=vcov.tssem1FEM(tssem1.obj),
-                                 n=sum(tssem1.obj$n), Amatrix=Amatrix, Smatrix=Smatrix, Fmatrix=Fmatrix,
+                                 n=sum(tssem1.obj$n), RAM=RAM, Amatrix=Amatrix, Smatrix=Smatrix, Fmatrix=Fmatrix,
                                  diag.constraints=diag.constraints, cor.analysis=tssem1.obj$cor.analysis,
                                  intervals.type=intervals.type, mx.algebras=mx.algebras,
                                  model.name=model.name, suppressWarnings=suppressWarnings,
@@ -537,7 +558,7 @@ tssem2 <- function(tssem1.obj, Amatrix=NULL, Smatrix=NULL, Fmatrix=NULL, diag.co
                       }
                       ## Use the original varible names for the observed covariance matrix
                       dimnames(pooledS) <- list(tssem1.obj$original.names, tssem1.obj$original.names)
-                      out <- wls(Cov=pooledS, aCov=aCov, n=tssem1.obj$total.n,
+                      out <- wls(Cov=pooledS, aCov=aCov, n=tssem1.obj$total.n, RAM=RAM,
                                  Amatrix=Amatrix, Smatrix=Smatrix, Fmatrix=Fmatrix, diag.constraints=diag.constraints,
                                  cor.analysis=cor.analysis, intervals.type=intervals.type, mx.algebras=mx.algebras,
                                  model.name=model.name, suppressWarnings = suppressWarnings, silent=silent, run=run, ...) })
